package net.minecraft.src;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import net.minecraft.client.Minecraft;

import org.lwjgl.opengl.GL11;

// SDK
public class mod_SdkGrapplingHook extends BaseMod
{
    public static Block blockRope;
    public static Block blockGrapplingHook;
    
    public static final Item itemRope = (new Item(2511)).setIconIndex(ModLoader.getUniqueSpriteIndex("/gui/items.png")).func_20011_a("itemRope");
    public static final Item itemGrapplingHook = (new SdkItemGrapplingHook(2512)).setIconIndex(ModLoader.getUniqueSpriteIndex("/gui/items.png")).func_20011_a("itemGrapplingHook");
    
    // Properties
    public static int blockIdRope = 115;
    public static int blockIdGrapplingHook = 116;

    // For sounds
    private static boolean soundsAdded = false;
  
	// Instead of in EntityPlayer
    public static Map<EntityPlayer, SdkEntityGrapplingHook> grapplingHooks = new HashMap<EntityPlayer, SdkEntityGrapplingHook>();
    
    static
    {
    	loadProperties();
    	initBlocks();
    }
	
    private static void loadProperties()
    {
    	Properties properties = new Properties();
    	try
    	{
    		String propertiesPath = mod_SdkFps.class.getProtectionDomain().getCodeSource().getLocation().toURI().getPath();
    		propertiesPath = propertiesPath.substring(0, propertiesPath.lastIndexOf('/') + 1);
    		propertiesPath += "mod_SdkGrapplingHook.properties";
    		
    		properties.load(new FileInputStream(propertiesPath));
    		String property;
    		
    		property = properties.getProperty("blockIdRope");
    		blockIdRope = Integer.parseInt(property.trim());
    		
    		property = properties.getProperty("blockIdGrapplingHook");
    		blockIdGrapplingHook = Integer.parseInt(property.trim());
        }
        catch(Exception e)
        {
        	System.out.println("Error loading properties.");
			e.printStackTrace();
        }
    }
    
    private static void initBlocks()
    {
    	blockRope = (new SdkBlockRope(blockIdRope, ModLoader.getUniqueSpriteIndex("/terrain.png"))).setHardness(-1F).setResistance(6000000F).setStepSound(Block.soundClothFootstep).func_20012_a("blockRope");
        blockGrapplingHook = (new SdkBlockGrapplingHook(blockIdGrapplingHook, ModLoader.getUniqueSpriteIndex("/terrain.png"))).setHardness(0.0F).setStepSound(Block.soundMetalFootstep).func_20012_a("blockGrapplingHook");
    }

	public mod_SdkGrapplingHook()
    {
    	ModLoader.RegisterBlock(blockRope);
    	ModLoader.RegisterBlock(blockGrapplingHook);
    	
    	ModLoader.AddName(blockRope, "Rope");
    	ModLoader.AddName(blockGrapplingHook, "Grappling Hook");
    	
    	ModLoader.AddName(itemRope, "Rope");
    	ModLoader.AddName(itemGrapplingHook, "Grappling Hook");
    	
		ModLoader.RegisterEntityID(SdkEntityGrapplingHook.class, "GrapplingHook", ModLoader.getUniqueEntityId());
		
		ModLoader.RegisterTileEntity(SdkTileEntityRope.class, "Rope");
    }

	@Override
	public String Version()
	{
		return "1.2_02v2";
	}
	
	@Override
	public void OSDHook(Minecraft minecraft, boolean menuOpen)
	{
		addSounds(minecraft);
	}
	
	private static void addSounds(Minecraft minecraft)
	{
		if(!soundsAdded)
		{
			soundsAdded = true;
			
			try
			{
				File resourcesFolder = new File(minecraft.mcDataDir, "resources/");
				
		        String[] soundTypes =  { "sound", "music", "streaming" };
		        for(int i = 0; i < soundTypes.length; i++)
		        {
		            File[] modSoundFiles = new File(resourcesFolder, "sdk/" + soundTypes[i]).listFiles();
		            for(int j = 0; j < modSoundFiles.length; j++)
		            	minecraft.installResource(soundTypes[i] + "/" + modSoundFiles[j].getName(), modSoundFiles[j]);
		        }
			}
			catch (Exception e)
			{
				System.out.println("Error adding sounds.");
				e.printStackTrace();
			}
		}
	}

	@Override
    public void AddRecipes(CraftingManager craftingmanager)
    {
		craftingmanager.addRecipe(new ItemStack(itemGrapplingHook, 1), new Object[] {
        	"X", "#", "#", Character.valueOf('#'), itemRope, Character.valueOf('X'), Item.ingotIron
        });
		craftingmanager.addRecipe(new ItemStack(itemRope, 1), new Object[] {
            "##", "##", "##", Character.valueOf('#'), Item.silk
        });
    }

	@Override
    public void AddRenderer(Map map)
    {
		map.put(SdkEntityGrapplingHook.class, new SdkRenderGrapplingHook());
    }

	@Override
    public void RegisterTextureOverrides()
    {
    	ModLoader.addOverride("/terrain.png", "/sdk/blockRope.png", blockRope.blockIndexInTexture);
    	ModLoader.addOverride("/terrain.png", "/sdk/blockGrapplingHook.png", blockGrapplingHook.blockIndexInTexture);
    	
        ModLoader.addOverride("/gui/items.png", "/sdk/itemRope.png", itemRope.iconIndex);
        ModLoader.addOverride("/gui/items.png", "/sdk/itemGrapplingHook.png", itemGrapplingHook.iconIndex);
    }
	
	@Override
	public void AddNetClientHandlerEntities(Map<Integer, Class> map)
	{
		map.put(119, SdkEntityGrapplingHook.class);
	}
    
	@Override
    public boolean RenderBlocks(RenderBlocks renderBlocks, Block block, int i,
			int j, int k, int l, IBlockAccess blockAccess,
			int overrideBlockTexture, boolean renderAllFaces)
	{
    	try
		{
	        if(l == blockRope.getRenderType())
	        {
	            return renderRope(block, i, j, k, blockAccess, overrideBlockTexture);
	        }
	        if(l == blockGrapplingHook.getRenderType())
	        {
	            return renderGrapplingHook(block, i, j, k, blockAccess, overrideBlockTexture, renderAllFaces);
	        }
		}
		catch (Exception e)
		{
			System.out.println("Error rendering mod_SdkGrapplingHook.");
			e.printStackTrace();
		}
		
        return false;
	}
	
    public static boolean renderRope(Block block, int i, int j, int k, IBlockAccess a, int b)
    {
        Tessellator tessellator = Tessellator.instance;
        int l = block.getBlockTextureFromSide(0);
        if(b >= 0)
        {
            l = b;
        }
        float f = block.getBlockBrightness(a, i, j, k);
        tessellator.setColorOpaque_F(f, f, f);
        int i1 = (l & 0xf) << 4;
        int j1 = l & 0xf0;
        double d = (float)i1 / 256F;
        double d1 = ((float)i1 + 15.99F) / 256F;
        double d2 = (float)j1 / 256F;
        double d3 = ((float)j1 + 15.99F) / 256F;
        int k1 = a.getBlockMetadata(i, j, k);
        float f1 = 0.0F;
        float f2 = 0.05F;
        if(k1 == 5)
        {
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 1) + f1, (float)(k + 1) + f1, d, d2);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 0) - f1, (float)(k + 1) + f1, d, d3);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 0) - f1, (float)(k + 0) - f1, d1, d3);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 1) + f1, (float)(k + 0) - f1, d1, d2);
            
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 0) - f1, (float)(k + 1) + f1, d1, d3);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 1) + f1, (float)(k + 1) + f1, d1, d2);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 1) + f1, (float)(k + 0) - f1, d, d2);
            tessellator.addVertexWithUV((float)i + f2, (float)(j + 0) - f1, (float)(k + 0) - f1, d, d3);
        }
        if(k1 == 4)
        {
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 1) + f1, (float)(k + 1) + f1, d, d2);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 0) - f1, (float)(k + 1) + f1, d, d3);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 0) - f1, (float)(k + 0) - f1, d1, d3);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 1) + f1, (float)(k + 0) - f1, d1, d2);
            
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 0) - f1, (float)(k + 1) + f1, d1, d3);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 1) + f1, (float)(k + 1) + f1, d1, d2);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 1) + f1, (float)(k + 0) - f1, d, d2);
            tessellator.addVertexWithUV((float)(i + 1) - f2, (float)(j + 0) - f1, (float)(k + 0) - f1, d, d3);
        }
        if(k1 == 3)
        {
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 0) - f1, (float)k + f2, d1, d3);
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 1) + f1, (float)k + f2, d1, d2);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 1) + f1, (float)k + f2, d, d2);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 0) - f1, (float)k + f2, d, d3);
            
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 1) + f1, (float)k + f2, d, d2);
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 0) - f1, (float)k + f2, d, d3);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 0) - f1, (float)k + f2, d1, d3);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 1) + f1, (float)k + f2, d1, d2);
        }
        if(k1 == 2)
        {
        	tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 0) - f1, (float)(k + 1) - f2, d1, d3);
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 1) + f1, (float)(k + 1) - f2, d1, d2);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 1) + f1, (float)(k + 1) - f2, d, d2);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 0) - f1, (float)(k + 1) - f2, d, d3);
            
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 1) + f1, (float)(k + 1) - f2, d, d2);
            tessellator.addVertexWithUV((float)(i + 1) + f1, (float)(j + 0) - f1, (float)(k + 1) - f2, d, d3);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 0) - f1, (float)(k + 1) - f2, d1, d3);
            tessellator.addVertexWithUV((float)(i + 0) - f1, (float)(j + 1) + f1, (float)(k + 1) - f2, d1, d2);
        }
        return true;
    }

    public static boolean renderGrapplingHook(Block block, int i, int j, int k, IBlockAccess a, int b, boolean d)
    {
        int l = block.colorMultiplier(a, i, j, k);
        float f = (float)(l >> 16 & 0xff) / 255F;
        float f1 = (float)(l >> 8 & 0xff) / 255F;
        float f2 = (float)(l & 0xff) / 255F;
        return renderGrapplingHook2(block, i, j, k, f, f1, f2, a, b, d);
    }
    
    public static boolean renderGrapplingHook2(Block block, int i, int j, int k, float f, float f1, float f2, IBlockAccess a, int b, boolean d)
    {
        Tessellator tessellator = Tessellator.instance;
        boolean flag = false;
        float f4 = 1.0F;
        float f7 = f4 * f;
        float f8 = f4 * f1;
        float f9 = f4 * f2;
        if(block == Block.grass)
        {
            f = f1 = f2 = 1.0F;
        }
        float f19 = block.getBlockBrightness(a, i, j, k);
        
        int metadata = a.getBlockMetadata(i, j, k);
        
        if(d || block.shouldSideBeRendered(a, i, j + 1, k, 1))
        {
            float f21 = block.getBlockBrightness(a, i, j + 1, k);
            if(block.maxY != 1.0D && !block.blockMaterial.getIsLiquid())
            {
                f21 = f19;
            }
            tessellator.setColorOpaque_F(f7 * f21, f8 * f21, f9 * f21);
            renderGrapplingHook3(block, i, j, k, block.getBlockTexture(a, i, j, k, 1), metadata, b);
            flag = true;
        }
        
        return flag;
    }
    
    public static void renderGrapplingHook3(Block block, double d, double d1, double d2, int i, int metadata, int b)
    {
        Tessellator tessellator = Tessellator.instance;
        if(b >= 0)
        {
            i = b;
        }
        int j = (i & 0xf) << 4;
        int k = i & 0xf0;
        double d3 = ((double)j + block.minX * 16D) / 256D;
        double d4 = (((double)j + block.maxX * 16D) - 0.01D) / 256D;
        double d5 = ((double)k + block.minZ * 16D) / 256D;
        double d6 = (((double)k + block.maxZ * 16D) - 0.01D) / 256D;
        if(block.minX < 0.0D || block.maxX > 1.0D)
        {
            d3 = ((float)j + 0.0F) / 256F;
            d4 = ((float)j + 15.99F) / 256F;
        }
        if(block.minZ < 0.0D || block.maxZ > 1.0D)
        {
            d5 = ((float)k + 0.0F) / 256F;
            d6 = ((float)k + 15.99F) / 256F;
        }
        double d7 = d + block.minX;
        double d8 = d + block.maxX;
        double d9 = d1 + block.maxY;
        double d10 = d2 + block.minZ;
        double d11 = d2 + block.maxZ;
        
        double temp;
        switch(metadata)
        {
	        case 2:
	        	// Vertical and horizontal flip
	        	temp = d5;
	        	d5 = d6;
	        	d6 = temp;
	        	
	        	temp = d3;
	        	d3 = d4;
	        	d4 = temp;
	        	break;
	        case 3:
	        	// Do nothing
	        	break;
	        case 4:
	        	// Vertical flip
	        	temp = d5;
	        	d5 = d6;
	        	d6 = temp;
	        	break;
	        case 5:
	        	// Horizontal flip
	        	temp = d3;
	        	d3 = d4;
	        	d4 = temp;
	        	break;
        }
        
        tessellator.addVertexWithUV(d8, d9, d11, d4, d6);
        tessellator.addVertexWithUV(d8, d9, d10, d4, d5);
        tessellator.addVertexWithUV(d7, d9, d10, d3, d5);
        tessellator.addVertexWithUV(d7, d9, d11, d3, d6);
    }
}
